package com.fjhx.utils;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.lang.Assert;
import com.alibaba.fastjson.JSONObject;
import com.fjhx.entity.EmailMailbox;
import com.fjhx.vo.*;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestTemplate;

import java.util.List;

@Component
public class EmailEngineUtil {

    private static String urlPrefix;

    private static final RestTemplate restTemplate = new RestTemplate();


    @Value("${config.urlPrefix}")
    public void setUrlPrefix(String urlPrefix) {
        EmailEngineUtil.urlPrefix = urlPrefix;
    }

    /**
     * 生成邮箱
     */
    public static void createAccount(BindingVo bindingVo) {
        // 验证邮件配置参数
        VerifyVo verifyVo = createVerifyVo(bindingVo);
        // 验证邮件配置是否正确
        VerifyResult verifyResult = post("v1/verifyAccount", verifyVo, VerifyResult.class);
        // 解析验证结果
        parsingVerifyResult(verifyResult);
        // 生成添加账号入参
        AddAccountVo addAccountVo = createAddAccountVo(verifyVo, bindingVo.getEmail());
        // 添加邮箱
        AddAccountResult addAccountResult = post("v1/account", addAccountVo, AddAccountResult.class);
        // 验证邮箱添加状态
        parsingAddAccountResult(addAccountResult);
    }

    /**
     * 查看邮箱所有文件夹
     */
    public static List<EmailMailbox> getMailboxList(String email) {
        String result = get("v1/account/" + email + "/mailboxes?counters=false", String.class);
        return BeanUtil.copyToList(JSONObject.parseObject(result).getJSONArray("mailboxes"), EmailMailbox.class);
    }


    /**
     * 生成验证邮箱配置实体
     */
    private static VerifyVo createVerifyVo(BindingVo bindingVo) {
        VerifyVo verifyVo = new VerifyVo();

        VerifyVo.Auth auth = new VerifyVo.Auth();
        auth.setUser(bindingVo.getEmail());
        auth.setPass(bindingVo.getSecretKey());

        VerifyVo.Tls tls = new VerifyVo.Tls();
        tls.setRejectUnauthorized(true);
        tls.setMinVersion("TLSv1.2");

        VerifyVo.Imap imap = new VerifyVo.Imap();
        imap.setAuth(auth);
        imap.setPort(bindingVo.getImapPort());
        imap.setHost(bindingVo.getImapHost());
        imap.setTls(tls);
        imap.setSecure(true);
        verifyVo.setImap(imap);

        VerifyVo.Smtp smtp = new VerifyVo.Smtp();
        smtp.setAuth(auth);
        smtp.setPort(bindingVo.getSmtpPort());
        smtp.setHost(bindingVo.getSmtpHost());
        smtp.setTls(tls);
        smtp.setSecure(true);
        verifyVo.setSmtp(smtp);

        return verifyVo;
    }

    /**
     * 解析验证结果
     */
    private static void parsingVerifyResult(VerifyResult verifyResult) {
        VerifyResult.Status imap = verifyResult.getImap();
        VerifyResult.Status smtp = verifyResult.getSmtp();

        Assert.notNull(imap, "imap解析结果为空");
        Assert.notNull(smtp, "smtp解析结果为空");

        Assert.isTrue(imap.getSuccess(), "imap解析失败：error：{} ；code：{}；", imap.getError(), imap.getCode());
        Assert.isTrue(smtp.getSuccess(), "smtp解析失败：error：{} ；code：{}；", smtp.getError(), smtp.getCode());
    }

    /**
     * 生成添加账号入参
     */
    private static AddAccountVo createAddAccountVo(VerifyVo verifyVo, String email) {
        AddAccountVo addAccountVo = BeanUtil.copyProperties(verifyVo, AddAccountVo.class);
        addAccountVo.setAccount(email);
        addAccountVo.setName(email);
        addAccountVo.setEmail(email);
        addAccountVo.setPath("*");
        return addAccountVo;
    }

    /**
     * 验证添加状态
     */
    private static void parsingAddAccountResult(AddAccountResult addAccountResult) {
        String state = addAccountResult.getState();

        Assert.isTrue("new".equals(state) || "existing".equals(state),
                "邮箱添加失败，account：{}，state：{}",
                addAccountResult.getAccount(), addAccountResult.getState());
    }

    public static <T> T get(String url, Class<T> cls) {
        return restTemplate.getForObject(urlPrefix + url, cls);
    }

    public static <T> T post(String url, Object paramObj, Class<T> cls) {
        return restTemplate.postForObject(urlPrefix + url, paramObj, cls);
    }

}
